! $id: diag_mod.f v1.0 2015/10/26 10:40:33 xxu unlvrtm v1.4.4
MODULE DIAG_MOD
!
 IMPLICIT NONE
!
! Make everthing PRIVATE,
 PRIVATE
!
! except
 PUBLIC  :: CREATE_DIAG
 PUBLIC  :: DEFINE_DIAG
 PUBLIC  :: WRITE_DIAG
 PUBLIC  :: CLOSEUP_DIAG

!
! Module variables
  INTEGER, PARAMETER   :: ncKind = 4
!
! All subroutines follow CONTAINS statement:
CONTAINS
!
!------------------------------------------------------------------------------
!
 SUBROUTINE CREATE_DIAG ( UInc )
!
! USES:
 USE NETCDF_MOD
 USE NAMELIST_ARRAY_MOD, ONLY : unlvrtmVersion
 USE NAMELIST_ARRAY_MOD, ONLY : Dir_Output
 USE NAMELIST_ARRAY_MOD, ONLY : Diag_Prefix 
 USE NAMELIST_ARRAY_MOD, ONLY : LVLIDORT, LJacob
 USE NAMELIST_ARRAY_MOD, ONLY : LEmiss_Atmos, LSolar
 USE NAMELIST_ARRAY_MOD, ONLY : LAerosol, LTracegas, LRayleigh
 USE NAMELIST_ARRAY_MOD, ONLY : LBRDF,    LLambertian
 USE NAMELIST_ARRAY_MOD, ONLY : n_Stoks
 USE ERROR_MOD,          ONLY : ERROR_STOP
 USE TIME_MOD,           ONLY : SYSTEM_TIMESTAMP

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(OUT)        :: UInc

! Local Variables
 LOGICAL, PARAMETER          :: DO_netCDF4 = .False.
 INTEGER                     :: vID
 CHARACTER(LEN=255)          :: ncFile
 CHARACTER(LEN=10 )          :: ncFormat  
 CHARACTER(LEN=16)           :: Stamp

!
! OUTPUT_DIAG begins here ...
!

! Define names of the output netCDF file and info file
 ncFile   = TRIM(Dir_Output) //TRIM(Diag_Prefix) // '.unlvrtm.nc'

! Create the netCDF file
 UInc = 99
 CALL netCDF_Create( UInc, ncFile, Write_NC4=DO_netCDF4 )
 
! Prepare some information for global attributes below

! netCDf format
 IF ( DO_netCDF4 ) THEN
    ncFormat = 'netCDF4'
 ELSE
    ncFormat = 'netCDF3'
 ENDIF 

! System time
 Stamp = SYSTEM_TIMESTAMP()

! -999 for Global attributes
 vID = -999
 CALL netCDF_Def_Attr( UInc, vID, 'UNL-VRTM Diagnostic File' , 'Title'   )
 CALL netCDF_Def_Attr( UInc, vID, 'Created on '//TRIM(Stamp), 'History' )
 CALL netCDF_Def_Attr( UInc, vID, TRIM(ncFormat)            , 'Format'  )
 CALL netCDF_Def_Attr( UInc, vID, 'UNL-VRTM'                , 'Model'   )
 CALL netCDF_Def_Attr( UInc, vID, TRIM(unlvrtmVersion)      , 'Version' )
 
! Model settings 
 IF ( LVLIDORT ) THEN 

    CALL netCDF_Def_Attr( UInc, vID, 'On',  'VLIDORT' )

    IF ( n_Stoks == 1 ) THEN
       CALL netCDF_Def_Attr( UInc, vID, 'Off',  'Vector' )
    ELSE
       CALL netCDF_Def_Attr( UInc, vID, 'On',  'Vector' )
    ENDIF

    IF ( LJacob ) THEN
       CALL netCDF_Def_Attr( UInc, vID, 'On',  'Jacobian' )
    ELSE
       CALL netCDF_Def_Attr( UInc, vID, 'Off', 'Jacobian' )
    ENDIF

    IF ( LSolar ) THEN
       CALL netCDF_Def_Attr( UInc, vID, 'On',  'Solar Radiation' )
    ELSE
       CALL netCDF_Def_Attr( UInc, vID, 'Off', 'Solar Radiation' )
    ENDIF

    IF ( LEmiss_Atmos ) THEN 
       CALL netCDF_Def_Attr( UInc, vID, 'On',  'Thermal Emission' )
    ELSE
       CALL netCDF_Def_Attr( UInc, vID, 'Off', 'Thermal Emission' )
    ENDIF

 ELSE
    CALL netCDF_Def_Attr( UInc, vID, 'Off', 'VLIDORT' )
 ENDIF

 IF ( LAerosol ) THEN
    CALL netCDF_Def_Attr( UInc, vID, 'On',  'Aerosol' )
 ELSE
    CALL netCDF_Def_Attr( UInc, vID, 'Off', 'Aerosol' )
 ENDIF

 IF ( LTracegas ) THEN 
    CALL netCDF_Def_Attr( UInc, vID, 'On',  'Tracegas' )
 ELSE
    CALL netCDF_Def_Attr( UInc, vID, 'Off', 'Tracegas' )
 ENDIF

 IF ( LRayleigh ) THEN
    CALL netCDF_Def_Attr( UInc, vID, 'On',  'Rayleigh' )
 ELSE
    CALL netCDF_Def_Attr( UInc, vID, 'Off', 'Rayleigh' )
 ENDIF

 IF ( LBRDF ) THEN 
    CALL netCDF_Def_Attr( UInc, vID, 'BRDF',  'Surface' )
 ELSE IF ( LLambertian ) THEN 
    CALL netCDF_Def_Attr( UInc, vID, 'Lambertian',  'Surface' )
 ENDIF

 ! Initialize as non-scucess
 CALL netCDF_Def_Attr( UInc, vID, 'No', 'Scucess' )

!
 RETURN
!
 END SUBROUTINE CREATE_DIAG
!
!------------------------------------------------------------------------------
!
 SUBROUTINE DEFINE_DIAG( UInc )
!
! USES:
! 
 USE NETCDF_MOD
 USE ATMPROF_MOD,        ONLY : n_Layers
 USE SPECTRA_MOD,        ONLY : nSpectra
 USE NAMELIST_ARRAY_MOD, ONLY : LFrequency
 USE NAMELIST_ARRAY_MOD, ONLY : n_Moments,    nGas,       nMode
 USE NAMELIST_ARRAY_MOD, ONLY : n_Theta0,     n_Theta,    n_Phi
 USE NAMELIST_ARRAY_MOD, ONLY : n_Geom,       nRecep_Lev, n_Stoks
 USE NAMELIST_ARRAY_MOD, ONLY : LTracegas,    LRayleigh
 USE NAMELIST_ARRAY_MOD, ONLY : LAerosol,     L_Refer_AOD
 USE NAMELIST_ARRAY_MOD, ONLY : LContinuum
 USE NAMELIST_ARRAY_MOD, ONLY : LBRDF,        n_BRDF
 USE NAMELIST_ARRAY_MOD, ONLY : LLambertian,  LSleave
 USE NAMELIST_ARRAY_MOD, ONLY : LVLIDORT
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob,       LJacob_Surf
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_Refr,  LJacob_Size
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_P,     LJacob_C
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_Gas
 USE NAMELIST_ARRAY_MOD, ONLY : LEmiss_Atmos
 USE NAMELIST_ARRAY_MOD, ONLY : LEmiss_Surface
 USE NAMELIST_ARRAY_MOD, ONLY : LDiag01,    LDiag02,   LDiag03, LDiag04
 USE NAMELIST_ARRAY_MOD, ONLY : LDiag05,    LDiag06,   LDiag07, LDiag08
 USE AEROSOL_MOD,        ONLY : NPangle
 USE LPAR_MOD,           ONLY : N_LPar_Total, LPar_Names
 USE LPAR_MOD,           ONLY : N_LPar_Surf
 USE IOP_MOD,            ONLY : nGMMask


!
 IMPLICIT NONE
!
 include 'netcdf.inc'
! 
! Arguments
 INTEGER, INTENT(IN)         :: UInc
!
! Local variables
 INTEGER                     :: Dim_Layer,  Dim_LayerRev 
 INTEGER                     :: Dim_Level,  Dim_LevelRev
 INTEGER                     :: Dim_Gas,    Dim_Spectra
 INTEGER                     :: Dim_SZA,    Dim_VZA,     Dim_RAZ,     Dim_LOut
 INTEGER                     :: Dim_Mode,   Dim_Geo,     Dim_Stokes
 INTEGER                     :: Dim_Stokes2
 INTEGER                     :: Dim_FMat,   Dim_PSDPar,  Dim_ProfPar
 INTEGER                     :: Dim_Moment, Dim_PAngle
 INTEGER                     :: Dim_LinPar, Dim_LinSPar
 INTEGER                     :: Dim_BRDFKernel
 INTEGER                     :: Dim_GMat
 INTEGER                     :: Dim_Ch6,    Dim_Ch10,    Dim_Ch30
 INTEGER                     :: Dim_N2,     Dim_N3,      Dim_N5

 INTEGER                     :: vID
 CHARACTER(LEN=40)           :: LName
 CHARACTER(LEN=20)           :: Units
 INTEGER                     :: Dims0D(0), Dims1D(1), Dims2D(2), Dims3D(3)
 INTEGER                     :: Dims4D(4), Dims5D(5), Dims6D(6)
 INTEGER                     :: N_Stoks2

!
! DEFINE_DIAG begins here ...
!

! Some initialization
 Dims0D(0:0) = (/ 0 /) 

! Define dimensions
 CALL netCDF_Def_Dim( UInc, 'Layer'   , N_Layers  , Dim_Layer   )
 CALL netCDF_Def_Dim( UInc, 'Level'   , N_Layers+1, Dim_Level   )
 CALL netCDF_Def_Dim( UInc, 'Spectra' , NSpectra  , Dim_Spectra ) 
 CALL netCDF_Def_Dim( UInc, 'Gas'     , NGas      , Dim_Gas     )
 CALL netCDF_Def_Dim( UInc, 'SZA'     , N_Theta0  , Dim_SZA     )
 CALL netCDF_Def_Dim( UInc, 'VZA'     , N_Theta   , Dim_VZA     )
 CALL netCDF_Def_Dim( UInc, 'RAZ'     , N_Phi     , Dim_RAZ     )
 CALL netCDF_Def_Dim( UInc, 'Mode'    , NMode     , Dim_Mode    )
 CALL netCDF_Def_Dim( UInc, 'Geo'     , N_Geom    , Dim_Geo     )
 CALL netCDF_Def_Dim( UInc, 'Stokes'  , N_Stoks   , Dim_Stokes  )
 CALL netCDF_Def_Dim( UInc, 'LOut'    , NRecep_Lev, Dim_LOut    )
 CALL netCDF_Def_Dim( UInc, 'PSDPar'  , 3         , dim_PSDPar  )
 CALL netCDF_Def_Dim( UInc, 'ProfPar' , 2         , dim_ProfPar )
 CALL netCDF_Def_Dim( UInc, 'Moment'  , n_Moments , Dim_Moment  )
 CALL netCDF_Def_Dim( UInc, 'FMat'    , 6         , Dim_FMat    )
 CALL netCDF_Def_Dim( UInc, 'PAngle'  , NPANGLE   , Dim_PAngle  )

 ! Stokes square
 IF ( N_Stoks == 1 ) THEN 
    N_Stoks2 = 1
 ELSE 
    N_Stoks2 = 16
 ENDIF
 CALL netCDF_Def_Dim( UInc, 'Stokes2', N_Stoks2, Dim_Stokes2  ) 

 ! BRDF 
 IF ( LBRDF ) THEN
    CALL netCDF_Def_Dim( UInc, 'BRDFKernel', N_BRDF, Dim_BRDFKernel )
 ENDIF

 ! Some dimensions is defined only when Jacobians are calculated
 IF ( LVLIDORT ) THEN 
    
    IF ( LJacob .AND. N_LPar_Total >= 1 ) THEN
       CALL netCDF_Def_Dim( UInc, 'LinPar', N_LPar_Total, Dim_LinPar )
    ENDIF 

    IF ( LJacob_Surf .AND. N_LPar_Surf >= 1 ) THEN
       CALL netCDF_Def_Dim( UInc, 'LinSPar', N_LPar_Surf, Dim_LinSPar )
    ENDIF 
 
    ! GMMask
    CALL netCDF_Def_Dim( UInc, 'GMat', nGMMask, Dim_GMat )  

    ! Reversed layer or level
    CALL netCDF_Def_Dim( UInc, 'LayerRev', N_Layers, Dim_LayerRev )
    CALL netCDF_Def_Dim( UInc, 'LevelRev', N_Layers+1, Dim_LevelRev )
  
 ENDIF

 ! Some un-specific dimensions
 CALL netCDF_Def_Dim( UInc, 'Ch6' , 6 , Dim_Ch6  )
 CALL netCDF_Def_Dim( UInc, 'Ch10', 10, Dim_Ch10 ) 
 CALL netCDF_Def_Dim( UInc, 'Ch30', 30, Dim_Ch30 ) 
 CALL netCDF_Def_Dim( UInc, 'Num2', 2 , Dim_N2   )
 CALL netCDF_Def_Dim( UInc, 'Num3', 3 , Dim_N3   )
 CALL netCDF_Def_Dim( UInc, 'Num5', 5 , Dim_N5   ) 

!
! Define variables
!

! Input variables
 IF ( LDIAG01 ) THEN 

    ! Lamdas: Spectral wavength
    vID    = 100
    LName  = 'Spectral wavength'
    Units  = 'nm'
    Dims1D = (/ Dim_Spectra /)
    CALL netCDF_Def_Var( UInc, 'Lamdas', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Wavenum: Spectral frequency
    vID    = vID + 1
    LName  = 'Spectral frequency'
    Units  = 'cm^-1'
    Dims1D = (/ Dim_Spectra /)
    CALL netCDF_Def_Var( UInc, 'Wavenum', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! FWHM: Full width at half maximum
    vID = vID + 1
    LName = 'Full-width at half-maximum'  
    IF ( LFrequency ) THEN 
      Units = 'cm^-1'
    ELSE 
      Units = 'nm'
    ENDIF 
    CALL netCDF_Def_Var( UInc, 'FWHM', NF_FLOAT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
  
    ! IAtm: Atmospheric profile type
    vID = vID + 1
    LName = 'Atmosphere type' 
    CALL netCDF_Def_Var( UInc, 'IAtm', NF_INT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )

    ! ZS: Altitude
    vID = vID + 1
    LName = 'Surface altitude'
    Units = 'm'
    CALL netCDF_Def_Var( UInc, 'ZS', NF_FLOAT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! PS: Surface pressure
    vID = vID + 1
    LName = 'Surface pressure'
    Units = 'hPa'
    CALL netCDF_Def_Var( UInc, 'PS', NF_FLOAT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! SZA: Solar zenith angle
    vID = vID + 1
    LName = 'Solar zenith angle'
    Units = 'Degree'
    Dims1D = (/ Dim_SZA /)
    CALL netCDF_Def_Var( UInc, 'SZA', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! VZA: View zenith angle
    vID = vID + 1
    LName = 'View zenith angle'
    Units = 'Degree'
    Dims1D = (/ Dim_VZA /)
    CALL netCDF_Def_Var( UInc, 'VZA', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! RAZ: Relative azimuthal angle
    vID = vID + 1
    LName = 'Relative azimuthal angle'
    Units = 'Degree'
    Dims1D = (/ Dim_RAZ /)
    CALL netCDF_Def_Var( UInc, 'RAZ', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! LOuts: Recptor Levels
    vID = vID + 1
    LName = 'Recptor level'
    Units = '#level'
    Dims1D = (/ Dim_LOut /)
    CALL netCDF_Def_Var( UInc, 'LOut', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! SSC: Single-scattering correction 
    vID = vID + 1
    LName = 'Single-scatter correction scenario'
    CALL netCDF_Def_Var( UInc, 'SSC', NF_INT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )

    ! S0: Solar flux factor
    vID = vID + 1
    LName = 'Solar flux factor'
    Units = 'none'
    CALL netCDF_Def_Var( UInc, 'S0', NF_FLOAT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! TS: Surface temperature
    vID = vID + 1
    LName = 'Surface temperature'
    Units = 'Kelvin'
    CALL netCDF_Def_Var( UInc, 'TS', NF_FLOAT, 0, Dims0D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Gas: Tracegas names
    vID = vID + 1
    LName = 'Tracegas names'
    Dims2D = (/ Dim_Ch6, Dim_Gas /)
    CALL netCDF_Def_Var( UInc, 'Gas', NF_CHAR, 2, Dims2D, vID )   
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )

    ! GasMW: Gas molecular weight
    vID = vID + 1
    LName = 'Tracegas mole weight'
    Units = 'g mole^-1'
    Dims1D = (/ Dim_Gas/)
    CALL netCDF_Def_Var( UInc, 'GasMW', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! GasSF: Tracegas scaling factor
    vID = vID + 1
    LName = 'Tracegas scaling factor'
    Units = 'none'
    Dims1D = (/ Dim_Gas/)
    CALL netCDF_Def_Var( UInc, 'GasSF', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! ModeFrc: Fractional modes
    vID = vID + 1
    LName = 'Modal fraction'
    Units = 'none'
    Dims1D = (/ Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'ModeFrc', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! MR: Real-part refractive index
    vID = vID + 1
    LName = 'Real-part refractive index'
    Units = 'none'
    Dims2D = (/ Dim_Spectra, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'MR', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! MI: Imag-part refractive index
    vID = vID + 1
    LName = 'Imaginary-part refractive index'
    Units = 'none'
    Dims2D = (/ Dim_Spectra, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'MI', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! PSDRange: Particle size range
    vID = vID + 1
    LName = 'Particle size range'
    Units = 'microns'
    Dims2D = (/ Dim_N2, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'PSDRange', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! PSDId: PSD index
    vID = vID + 1
    LName = 'PSD index'
    Units = 'none'
    Dims1D = (/ Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'PSDId', NF_INT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! PSDPars: PSD parameters
    vID = vID + 1
    LName = 'PSD parameters'
    Units = 'vary'
    Dims2D = (/ Dim_PSDPar, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'PSDPars', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! ProfRange: Aerosol profile range
    vID = vID + 1
    LName = 'Aerosol vertical extending range'
    Units = 'km'
    Dims2D = (/ Dim_N2, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'ProfRange', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! ProfId: Aerosol profile type
    vID = vID + 1
    LName = 'Aerosol vertical profile type'
    Units = 'none'
    Dims1D = (/ Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'ProfId', NF_INT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! ProfPars: Aerosol profile parameters 
    vID = vID + 1
    LName = 'Aerosol vertical profile parameters'
    Units = 'vary'
    Dims2D = (/ Dim_ProfPar, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'ProfPars', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! For reference aerosol mode
    IF ( L_REFER_AOD ) THEN

       ! Ref_Lamda
       vID = vID + 1
       LName = 'Reference wavelength'
       Units = 'nm'
       CALL netCDF_Def_Var( UInc, 'Ref_Lamda', NF_FLOAT, 0, Dims0D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Ref_MR
       vID = vID + 1
       LName = 'Reference real refractive index'
       Units = 'none' 
       CALL netCDF_Def_Var( UInc, 'Ref_MR', NF_FLOAT, 0, Dims0D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Ref_MI
       vID = vID + 1
       LName = 'Referenced imaginary refractive index'
       Units = 'none'
       CALL netCDF_Def_Var( UInc, 'Ref_MI', NF_FLOAT, 0, Dims0D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Ref_PSDId
       vID = vID + 1
       LName = 'Referenced PSD index'
       Units = 'none'
       CALL netCDF_Def_Var( UInc, 'Ref_PSDId', NF_INT, 0, Dims0D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Ref_PSDRange
       vID = vID + 1
       LName = 'Referenced PSD range'
       Units = 'microns'
       Dims1D = (/ Dim_N2 /)
       CALL netCDF_Def_Var( UInc, 'Ref_PSDRange', NF_FLOAT, 1, Dims1D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Ref_PSDPars
       vID = vID + 1
       LName = 'Referenced PSD parameters'
       Units = 'vary'
       Dims1D = (/ Dim_N3 /)
       CALL netCDF_Def_Var( UInc, 'Ref_PSDPars', NF_FLOAT, 1, Dims1D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
 
    ENDIF ! End of L_REFER_AOD

    ! Jacobian parameter names
    IF ( LVLIDORT .AND. LJacob .AND. N_LPar_Total >= 1 ) THEN 

       vID = vID + 1
       LName  = 'Jacobian names'
       Dims2D = (/ Dim_Ch30, Dim_LinPar /)
       CALL netCDF_Def_Var( UInc, 'LinPar', NF_CHAR, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )

    ENDIF

 ENDIF ! End of LDIAG01

! Atmospheric profile variables
 IF ( LDIAG02 ) THEN 

    ! Z: Level height
    vID    = 200
    LName  = 'Level height'
    Units  = 'km'
    Dims1D = (/ Dim_Level /)
    CALL netCDF_Def_Var( UInc, 'Z', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
   
    ! P: Level Pressure
    vID = vID + 1
    LName = 'Level pressure'
    Units = 'hPa'
    Dims1D = (/ Dim_Level /)    
    CALL netCDF_Def_Var( UInc, 'P', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )


    ! T: Level Temperature
    vID = vID + 1
    LName = 'Level temperature'
    Units = 'Kelvin'
    Dims1D = (/ Dim_Level /)
    CALL netCDF_Def_Var( UInc, 'T', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! LayP: Layer pressure
    vID = vID + 1
    LName = 'Layer pressure'
    Units = 'hPa'
    Dims1D = (/ Dim_Layer /)
    CALL netCDF_Def_Var( UInc, 'LayP', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! LayT: Layer temperature
    vID = vID + 1
    LName = 'Layer temperature'
    Units = 'Kelvin'
    Dims1D = (/ Dim_Layer /)
    CALL netCDF_Def_Var( UInc, 'LayT', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! AirDen: Layer air number density
    vID = vID + 1 
    LName = 'Air number density'
    Units = 'cm^-3'
    Dims1D = (/ Dim_Layer /)
    CALL netCDF_Def_Var( UInc, 'AirDen', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! GasMR: Gaseous mixing ratio
    vID = vID + 1
    LName = 'Tracegas mixing ratio'
    Units = 'ppm'
    Dims2D = (/ Dim_Layer, Dim_Gas /)
    CALL netCDF_Def_Var( UInc, 'GasMR', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Aerosol vertical profiles
    IF ( LAerosol ) THEN 

       ! ProfVol: Aerosol volume profile
       vID    = vID + 1
       LName  = 'Aerosol volume profile'
       Units  = 'um^3/um^2'
       Dims2D = (/ Dim_Layer, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'AerVol', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! PROF_Num: Particle number profile
       vID    = vID + 1
       LName  = 'Aerosol number profile'
       Units  = 'um^-2'
       Dims2D = (/ Dim_Layer, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'AerNum', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

 ENDIF ! ENd of LDIAG02

 ! Aerosol scattering properties
 IF ( LDiag03 .AND.  LAerosol ) THEN 
  
    ! PAngles
    vID    = 300
    LName  = 'Equidistant scattering angles'
    Units  = 'Degree'
    Dims1D = (/ Dim_PAngle /)
    CALL netCDF_Def_Var( UInc, 'PAngle', NF_FLOAT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! BULK_AOP: Mie/Tmatrix bulk parameters
    vID    = vID + 1
    LName  = 'Mie/Tmatrix bulk parameters'
    Units  = 'vary'
    Dims3D = (/ Dim_Spectra, Dim_N5, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'Bulk_AOP', NF_FLOAT, 3, Dims3D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Asymm: Asymmetric factor
    vID    = vID + 1
    LName  = 'Asymmetric factor'
    Units  = 'None'
    Dims2D = (/ Dim_Spectra, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'Asymm', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Dist: Distribution parameters
    vID    = vID + 1
    LName  = 'Size distribution parameters'
    Units  = 'vary'
    Dims2D = (/ Dim_N5, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'Dist', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! ExpCoeffs: Greek matrix expansion coefficients
    vID    = vID + 1
    LName  = 'Greek matrix expansion coefficients'
    Units  = 'none'
    Dims4D = (/ Dim_Spectra, Dim_FMat, Dim_Moment, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'ExpCoeffs', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! FMatrix: Scattering F-matrix
    vID    = vID + 1
    LName  = 'Scattering F-matrix'
    Units  = 'none'
    Dims4D = (/ Dim_Spectra, Dim_FMat, Dim_PAngle, Dim_Mode /)
    CALL netCDF_Def_Var( UInc, 'FMatrix', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    IF ( LJacob_Size ) THEN 

       ! LPSD_Bulk: Bulk_AOP derivatives wrt PSD
       vID    = vID + 1
       LName  = 'Bulk_AOP derivatives wrt PSD'
       Units  = 'vary'
       Dims4D = (/ Dim_Spectra, Dim_N5, Dim_PSDPar, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'LPSD_Bulk', NF_FLOAT, 4, Dims4D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! LPSD_Dist: PSDPars derivatives wrt PSD
       vID    = vID + 1
       LName  = 'PSDPars derivatives wrt PSD'
       Units  = 'vary'
       Dims3D = (/ Dim_N5, Dim_PSDPar, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'LPSD_Dist', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! LPSD_FMatrix: FMatrix derivatives wrt PSD
       vID    = vID + 1
       LName  = 'FMatrix derivatives wrt PSD'
       Units  = 'vary'
       Dims5D = (/ Dim_Spectra, Dim_FMat, Dim_PAngle, Dim_PSDPar, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'LPSD_FMatrix', NF_FLOAT, 5, Dims5D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

    IF ( LJacob_Refr ) THEN

       ! LRFE_Bulk: Bulk_AOP derivatives wrt refractive index
       vID    = vID + 1
       LName  = 'Bulk_AOP derivatives wrt refractive index'
       Units  = 'vary'
       Dims4D = (/ Dim_Spectra, Dim_N5, Dim_N3, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'LRFE_Bulk', NF_FLOAT, 4, Dims4D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! LRFE_FMatrix: FMatrix derivatives wrt refractive index
       vID    = vID + 1
       LName  = 'FMatrix derivatives wrt refractive index'
       Units  = 'vary'
       Dims5D = (/ Dim_Spectra, Dim_FMat, Dim_PAngle, Dim_N3, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'LRFE_FMatrix', NF_FLOAT, 5, Dims5D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

    ! LFRC_Bulk: Bulk_AOP derivatives wrt modal fraction
    IF ( nMode == 2 ) THEN 
       vID    = vID + 1
       LName  = 'Bulk_AOP derivatives wrt modal fraction'
       Units  = 'vary'
       Dims3D = (/ Dim_Spectra, Dim_N5, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'LFrc_Bulk', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
    ENDIF 

 ENDIF ! End of LDIAG03

 ! Optical propfiles: Aerosol, Rayleigh, and Tracegas optical properties
 IF ( LDIAG04 ) THEN 

    vID =  400

    ! Only do the output if the Trace gas calculation is turned on
    IF ( LTracegas ) THEN

       ! Xsec_SAO: XSEC_SAO(NSPECTRA,N_LAYERS,NGAS)
       vID    = vID + 1
       LName  = 'Gas-absorption cross-section (SAO)'
       Units  = 'cm^2/molec'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
       CALL netCDF_Def_Var( UInc, 'Xsec_SAO', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Xsec_LBL: XSEC_HIT(NSPECTRA,N_LAYERS,NGAS)
       vID    = vID + 1
       LName  = 'Gas-absorption cross-section (HITRAN)'
       Units  = 'cm^2/molec'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
       CALL netCDF_Def_Var( UInc, 'Xsec_LBL', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Xsec_Combined: GAS_XSEC(NSPECTRA,N_LAYERS,NGAS)
       vID    = vID + 1
       LName  = 'Gas-absorption cross-section combined'
       Units  = 'cm^2/molec'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
       CALL netCDF_Def_Var( UInc, 'Xsec_Combined', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! GasOD_SAO: XABS_SAO(NSPECTRA,N_LAYERS,NGAS)
       vID    = vID + 1
       LName  = 'Gas-absorption optical depth (SAO)'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
       CALL netCDF_Def_Var( UInc, 'tauGas_SAO', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! GasOD_LBL: XABS_HIT(NSPECTRA,N_LAYERS,NGAS)
       vID    = vID + 1
       LName  = 'Gas-absorption optical depth (HITRAN)'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
       CALL netCDF_Def_Var( UInc, 'tauGas_LBL', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! GasOD_Combined: GAS_XSEC(NSPECTRA,N_LAYERS,NGAS)
       vID    = vID + 1
       LName  = 'Gas-absorption optical depth combined'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
       CALL netCDF_Def_Var( UInc, 'tauGas_Combined', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Continnum
       IF ( LCONTINUUM ) THEN 

          ! Xsec_CTN: XSEC_CTN(NSPECTRA,N_LAYERS,NGAS)
          vID    = vID + 1
          LName  = 'Gas-absorption cross-section (Continuum)'
          Units  = 'cm^2/molec'
          Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
          CALL netCDF_Def_Var( UInc, 'Xsec_CTN', NF_FLOAT, 3, Dims3D, vID )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

          ! GasOD_CTN: XABS_CTN(NSPECTRA,N_LAYERS,NGAS)
          vID    = vID + 1
          LName  = 'Gas-absorption optical depth (Continuum)'
          Units  = 'none'
          Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Gas /)
          CALL netCDF_Def_Var( UInc, 'tauGas_CTN', NF_FLOAT, 3, Dims3D, vID )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ENDIF ! LCONTINUUM

       ! GasOD: the sum of all gases
       vID    = vID + 1
       LName  = 'Bulk gas-absorption optical depth'
       Units  = 'none'
       Dims2D = (/ Dim_Spectra, Dim_Layer /)
       CALL netCDF_Def_Var( UInc, 'tauGas', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF ! End of LTRACEGAS

    ! Only do the output if the Rayleigh calculation is turned on
    IF ( LRayleigh ) THEN

       ! RAYLEIGH_XSCA(NSPECTRA,N_LAYERS)
       vID    = vID + 1
       LName  = 'Rayleigh-scatting optical depth'
       Units  = 'none'
       Dims2D = (/ Dim_Spectra, Dim_Layer /)
       CALL netCDF_Def_Var( UInc, 'tauRayleigh', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! RAYLEIGH_PHAS(NSPECTRA,0:2,6)
       vID    = vID + 1
       LName  = 'Rayleigh-scatting Greek matrix'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_N3, Dim_FMat /)
       CALL netCDF_Def_Var( UInc, 'Rayleigh_FMatrix', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF ! End of LRayleigh

    ! Only do the output if the aerosol is turned on
    IF ( LAerosol ) THEN 

       ! tauAER
       vID    = vID + 1
       LName  = 'Aerosol optical depth'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'tauAER', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! omegaAER
       vID    = vID + 1
       LName  = 'Aerosol single scattering albedo'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_Layer, Dim_Mode /)
       CALL netCDF_Def_Var( UInc, 'omegaAER', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF ! End of LAerosol

 ENDIF ! End of LDIAG04

 ! Surface properties
 IF ( LDiag05 ) THEN 

    vID = 500

    ! Lambertian Albedo
    IF ( LLambertian ) THEN 

       vID = vID
       LName  = 'Lambertian albedo'
       Units  = 'none'
       Dims1D = (/ Dim_Spectra /)
       CALL netCDF_Def_Var( UInc, 'Lamb_Albedo', NF_FLOAT, 1, Dims1D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

    ! Exact direct BRDF calculated by VBRDF_SUP
    IF ( LBRDF ) THEN 

       ! BRDFKernel
       vID = vID + 1
       LName = 'BRDF kernel names'
       Dims2D = (/ Dim_Ch10, Dim_BRDFKernel /)
       CALL netCDF_Def_Var( UInc, 'BRDFKernel', NF_CHAR, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )

       ! BRDFFactor
       vID    = vID + 1
       LName  = 'BRDF coefficients'
       Units  = 'none'
       Dims2D = (/ Dim_Spectra, Dim_BRDFKernel /)
       CALL netCDF_Def_Var( UInc, 'BRDFFactor', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! BRDFVAL(N_STOKS,N_THETA,N_PHI,N_THETA0)
       vID    = vID + 1
       LName  = 'Exact direct BRDF values'
       Units  = 'none'
       Dims5D = (/ Dim_Spectra, Dim_Stokes2, Dim_VZA, Dim_RAZ, Dim_SZA /)
       CALL netCDF_Def_Var( UInc, 'BRDFVal', NF_FLOAT, 5, Dims5D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
  
       IF ( LJacob_Surf ) THEN 

          ! BRDFJacob
          vID    = vID + 1
          LName  = 'Jacobians of Exact direct BRDF with kernel factors'
          Units  = 'none'
          Dims6D = (/ Dim_Spectra, Dim_LinSPar, Dim_Stokes2, Dim_VZA, Dim_RAZ, Dim_SZA /)
          CALL netCDF_Def_Var( UInc, 'BRDFJacob', NF_FLOAT, 6, Dims6D, vID )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ENDIF

    ENDIF

    ! Isotropic surface leaving term
    IF ( LSleave ) THEN 

       ! SLeaveVal
       vID    = vID + 1
       LName  = 'Isotropic surface-leaving radiance'
       Units  = 'none'
       Dims3D = (/ Dim_Spectra, Dim_Stokes, Dim_SZA /)
       CALL netCDF_Def_Var( UInc, 'SLeaveVal', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
     
    ENDIF

 ENDIF ! End of LDIAG05

 ! Inputs of optical properties for VLIDORT
 IF ( LDiag06 .AND. LVLIDORT ) THEN 

    ! GMat
    vID    = 600
    LName  = 'Masking index for Greek matrix'
    Units  = 'none'
    Dims1D = (/ Dim_GMat /)
    CALL netCDF_Def_Var( UInc, 'GMat', NF_INT, 1, Dims1D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! tauIOP
    vID    = vID + 1
    LName  = 'Bulk atmospheric optical depth'
    Units  = 'none'
    Dims2D = (/Dim_Spectra, Dim_LayerRev /)
    CALL netCDF_Def_Var( UInc, 'tauIOP', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! omegaIOP
    vID    = vID + 1
    LName  = 'Bulk atmospheric single scattering albedo'
    Units  = 'none'
    Dims2D = (/Dim_Spectra, Dim_LayerRev /)
    CALL netCDF_Def_Var( UInc, 'omegaIOP', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! GMatIOP
    vID    = vID + 1
    LName  = 'Bulk atmospheric scattering Greek matrix'
    Units  = 'none'
    Dims4D = (/Dim_Spectra, Dim_Moment, Dim_LayerRev, Dim_Gmat/)
    CALL netCDF_Def_Var( UInc, 'GMatIOP', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    IF ( LJacob .AND. N_LPar_Total >= 1 ) THEN 

       ! tauLIOP
       vID    = vID + 1
       LName  = 'tauIOP linearized inputs'
       Units  = 'none'
       Dims3D = (/Dim_Spectra, Dim_LinPar, Dim_LayerRev /)
       CALL netCDF_Def_Var( UInc, 'tauLIOP', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! omegaLIOP
       vID    = vID + 1
       LName  = 'omegaIOP linearized inputs'
       Units  = 'none'
       Dims3D = (/Dim_Spectra, Dim_LinPar, Dim_LayerRev /)
       CALL netCDF_Def_Var( UInc, 'omegaLIOP', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! GMatLIOP
       vID    = vID + 1
       LName  = 'GMatIOP linearized inputs'
       Units  = 'none'
       Dims5D = (/Dim_Spectra, Dim_LinPar, Dim_Moment, Dim_LayerRev, Dim_Gmat/)
       CALL netCDF_Def_Var( UInc, 'GMatLIOP', NF_FLOAT, 5, Dims5D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

 ENDIF ! END of LDIAG06

 ! VLIDORT calculations
 IF ( LDiag07 .AND. LVLIDORT ) THEN

    ! VZA Offset
    vID = 700
    LName  = 'VZA offset'
    Units  = 'none'
    Dims2D = (/ Dim_SZA, Dim_VZA /)
    CALL netCDF_Def_Var( UInc, 'Geo_Offset', NF_FLOAT, 2, Dims2D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Stokes vector
    vID = vID + 1
    LName  = 'Stokes vector'
    Units  = 'sr^-1'
    Dims4D = (/ Dim_Spectra, Dim_LOut, Dim_Geo, Dim_Stokes /)
    CALL netCDF_Def_Var( UInc, 'Stokes', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Stokes mean vector
    vID    = vID + 1
    LName  = 'Stokes mean vector (actinic flux)'
    Units  = 'none'
    Dims4D = (/ Dim_Spectra, Dim_LOut, Dim_SZA, Dim_Stokes /)
    CALL netCDF_Def_Var( UInc, 'Mean_Stokes', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Stokes flux vector
    vID    = vID + 1
    LName  = 'Stokes flux vector (regular flux)'
    Units  = 'none'
    Dims4D = (/ Dim_Spectra, Dim_LOut, Dim_SZA, Dim_Stokes /)
    CALL netCDF_Def_Var( UInc, 'Flux_Stokes', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Stokes direct mean vector
    vID    = vID + 1
    LName  = 'Stokes direct mean vector (actinic flux)'
    Units  = 'none'
    Dims4D = (/ Dim_Spectra, Dim_LOut, Dim_SZA, Dim_Stokes /)
    CALL netCDF_Def_Var( UInc, 'Mean_Direct', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Stokes direct flux vector
    vID    = vID + 1
    LName  = 'Stokes direct flux vector (regular flux)'
    Units  = 'none'
    Dims4D = (/ Dim_Spectra, Dim_LOut, Dim_SZA, Dim_Stokes /)
    CALL netCDF_Def_Var( UInc, 'Flux_Direct', NF_FLOAT, 4, Dims4D, vID )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
    CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ! Surface emission related 
    IF ( LEmiss_Surface ) THEN

       ! Surface thermal emission
       vID    = vID + 1
       LName  = 'Surface thermal emission'
       IF ( LFrequency ) THEN
          Units  = 'W/m2/sr/cm^-1'
       ELSE 
          Units  = 'W/m2/sr/um'
       ENDIF
       Dims1D = (/ Dim_Spectra /)
       CALL netCDF_Def_Var( UInc, 'Emiss_Surface', NF_FLOAT, 1, Dims1D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )  

       ! Gradient Emiss_Surface wrt surface T'
       vID    = vID + 1
       LName  = 'Gradient Emiss_Surface wrt surface T'
       IF ( LFrequency ) THEN
          Units  = 'W/m2/sr/cm^-1/K'
       ELSE
          Units  = 'W/m2/sr/um/K'
       ENDIF       
       Dims1D = (/ Dim_Spectra /)
       CALL netCDF_Def_Var( UInc, 'GDT_Emiss_Surface', NF_FLOAT, 1, Dims1D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

    ! Atmosphere emissions
    IF ( LEmiss_Atmos ) THEN

       ! Air thermal emission
       vID    = vID + 1
       LName  = 'Air thermal emission' 
       IF ( LFrequency ) THEN
          Units  = 'W/m2/sr/cm^-1'
       ELSE
          Units  = 'W/m2/sr/um'
       ENDIF
       Dims2D = (/ Dim_Spectra, Dim_LevelRev /)
       CALL netCDF_Def_Var( UInc, 'Emiss_Atmos', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ! Gradient Emiss_Surface wrt surface T'
       vID    = vID + 1
       LName  = 'Gradient Eimss_Atmos wrt air T'
       IF ( LFrequency ) THEN
          Units  = 'W/m2/sr/cm^-1/K'
       ELSE
          Units  = 'W/m2/sr/um/K'
       ENDIF
       Dims2D = (/ Dim_Spectra, Dim_LevelRev /)
       CALL netCDF_Def_Var( UInc, 'GDT_Emiss_Atmos', NF_FLOAT, 2, Dims2D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

 ENDIF ! End if LDiag07

 ! Atmosphere Jacobians from VLIDORT
 vID = 800
 IF ( LDiag08 .AND. LVLIDORT .AND. LJacob ) THEN 
 
    IF ( N_LPar_Total >= 1 ) THEN 

       ! Columnar Jacobians
       IF ( LJACOB_C ) THEN 
          vID = vID + 1
          LName  = 'Columnar Jacobians'
          Units  = 'sr^-1'
          Dims5D = (/ Dim_Spectra, Dim_LinPar, Dim_LOut, Dim_Geo, Dim_Stokes /)
          CALL netCDF_Def_Var( UInc, 'Jacob_Column', NF_FLOAT, 5, Dims5D, vID )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
       ENDIF 
       
       ! Profile Jacobians
       IF ( LJACOB_P ) THEN  
          vID = vID + 1
          LName  = 'Profile Jacobians'
          Units  = 'sr^-1'
          Dims6D = (/ Dim_Spectra, Dim_LinPar, Dim_LayerRev, Dim_LOut, Dim_Geo, Dim_Stokes /)
          CALL netCDF_Def_Var( UInc, 'Jacob_Prof', NF_FLOAT, 6, Dims6D, vID )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
       ENDIF 

       ! Jacobian to gas profile
       IF ( LJacob_Gas .AND. LJACOB_P ) THEN 

          vID = vID + 1
          LName  = 'Gaseous Profile Jacobians'
          Units  = 'sr^-1' 
          Dims6D = (/ Dim_Spectra, Dim_Gas, Dim_LayerRev, Dim_LOut, Dim_Geo,Dim_Stokes /)
          CALL netCDF_Def_Var( UInc, 'Jacob_Prof_Gas', NF_FLOAT, 6, Dims6D, vID )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
          CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

       ENDIF

    ENDIF

    ! AOD Jacobians with respect to aerosol parameters
    IF ( LAerosol .AND. N_LPar_Total >= 1 ) THEN 
       vID = vID + 1
       LName  = 'AOD profile Jacobians'
       Units  = 'vary'
       Dims3D = (/ Dim_Spectra, Dim_LinPar, Dim_Layer /)
       CALL netCDF_Def_Var( UInc, 'tauAER_WFS', NF_FLOAT, 3, Dims3D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )
    ENDIF

    ! Surface Jacobians from VLIDOR
    IF ( LJACOB_SURF .AND. N_LPar_Surf >= 1 ) THEN 

       vID = vID + 1
       LName  = 'Surface Jacobians'
       Units  = 'sr^-1'
       Dims5D = (/ Dim_Spectra, Dim_LinSPar, Dim_LOut, Dim_Geo, Dim_Stokes /)
       CALL netCDF_Def_Var( UInc, 'Jacob_Surface', NF_FLOAT, 5, Dims5D, vID )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Lname), 'longname' )
       CALL netCDF_Def_Attr( UInc, vID, TRIM(Units), 'units'    )

    ENDIF

 ENDIF

!
! End of define mode
!
 CALL netCDF_End_Def( UInc )

!
 RETURN
!
 END SUBROUTINE DEFINE_DIAG
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG( UInc, iLam )
!
! USES:
 USE NAMELIST_ARRAY_MOD, ONLY : LDiag01, LDiag02, LDiag03, LDiag04
 USE NAMELIST_ARRAY_MOD, ONLY : LDiag05, LDiag06, LDiag07, LDiag08
 USE NAMELIST_ARRAY_MOD, ONLY : LVLIDORT, LAerosol, LJacob

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variables
 LOGICAL, SAVE              :: First = .True.


!
! WRITE_DIAG_NonSpectra begins here ...
!

! Variables only write once at the first spectra loop
 IF ( iLam == 1 .OR. First ) THEN 

    ! Write out the model inputs
    IF ( LDiag01 ) CALL WRITE_DIAG01( UInc )

    ! Write out atmospheric profiles
    IF ( LDiag02 ) CALL WRITE_DIAG02( UInc )

    ! Change First to False 
    First = .False.

 ENDIF

! Variables need to be writen for each spectra:

 ! Mie properties and their linearization  
 IF ( LDiag03 .AND. LAerosol ) CALL WRITE_DIAG03( UInc, iLam )

 ! Atmospheric optical profiles, ie, gas, rayleigh, and aeorsol
 IF ( LDiag04 ) CALL WRITE_DIAG04( UInc, iLam ) 

 ! Surface properties
 IF ( LDiag05 ) CALL WRITE_DIAG05( UInc, iLam )

 ! Following outputs rely on VLIDORT
 IF ( LVLIDORT ) THEN 

    IF ( LDiag06 ) CALL WRITE_DIAG06( UInc, iLam )
    IF ( LDiag07 ) CALL WRITE_DIAG07( UInc, iLam )
    IF ( LDiag08 .AND. LJacob ) CALL WRITE_DIAG08( UInc, iLam )

 ENDIF

 END SUBROUTINE WRITE_DIAG
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG01( UInc )
!
! USES:
 USE NETCDF_MOD
 USE SPECTRA_MOD,        ONLY : NSpectra,      Lamdas
 USE SPECTRA_MOD,        ONLY : WaveNums
 USE SPECTRA_MOD,        ONLY : NReal0,      NImag0
 USE NAMELIST_ARRAY_MOD, ONLY : Spectra_FWHM
 USE NAMELIST_ARRAY_MOD, ONLY : IATMOS
 USE NAMELIST_ARRAY_MOD, ONLY : ALTITUDE, SURF_PRESS
 USE NAMELIST_ARRAY_MOD, ONLY : Theta0, n_Theta0
 USE NAMELIST_ARRAY_MOD, ONLY : Theta,  n_Theta
 USE NAMELIST_ARRAY_MOD, ONLY : Phi,    n_Phi
 USE NAMELIST_ARRAY_MOD, ONLY : Recep_Lev, Nrecep_Lev
 USE NAMELIST_ARRAY_MOD, ONLY : I_Scenario
 USE NAMELIST_ARRAY_MOD, ONLY : Solar_Flux
 USE NAMELIST_ARRAY_MOD, ONLY : Surface_Temp
 USE NAMELIST_ARRAY_MOD, ONLY : Gasname, nGas
 USE NAMELIST_ARRAY_MOD, ONLY : GasMu,   GasSF
 USE NAMELIST_ARRAY_MOD, ONLY : ModeFrc, nMode
 USE NAMELIST_ARRAY_MOD, ONLY : IDist, IDProf
 USE NAMELIST_ARRAY_MOD, ONLY : SizeRange, Prof_Range
 USE NAMELIST_ARRAY_MOD, ONLY : DistPar, Prof_Par
 USE NAMELIST_ARRAY_MOD, ONLY : Refer_Lamda
 USE NAMELIST_ARRAY_MOD, ONLY : Refer_RMRI
 USE NAMELIST_ARRAY_MOD, ONLY : Refer_IDist
 USE NAMELIST_ARRAY_MOD, ONLY : Refer_DistPar
 USE NAMELIST_ARRAY_MOD, ONLY : Refer_SizeRange

 USE NAMELIST_ARRAY_MOD, ONLY : LDiag01, LDiag02, LDiag03, LDiag04
 USE NAMELIST_ARRAY_MOD, ONLY : LDiag05, LDiag06, LDiag07, LDiag08
 USE NAMELIST_ARRAY_MOD, ONLY : L_Refer_AOD
 USE NAMELIST_ARRAY_MOD, ONLY : LVLIDORT, LJacob
 USE LPAR_MOD,           ONLY : n_LPar_Total, LPar_Names 

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
!
! Local variables
 INTEGER            :: st1D, st2D(2)
 INTEGER            :: ct1D, ct2D(2)
 INTEGER            :: StrLen

!
! WRITE_DIAG01 begins here ...
!

 CALL netCDF_Write_Var( UInc, REAL(Lamdas,ncKind),   'Lamdas', 1,  NSpectra )
 CALL netCDF_Write_Var( UInc, REAL(WaveNums,ncKind), 'Wavenum', 1, NSpectra )
 CALL netCDF_Write_Var( UInc, REAL(Spectra_FWHM, ncKind),    'FWHM' ) 
 CALL netCDF_Write_Var( UInc, IAtmos,'IAtm' ) 
 CALL netCDF_Write_Var( UInc, ALTITUDE,'ZS' ) 
 CALL netCDF_Write_Var( UInc, SURF_PRESS,'PS' ) 
 CALL netCDF_Write_Var( UInc, REAL(Theta0,ncKind),  'SZA', 1, n_Theta0 )
 CALL netCDF_Write_Var( UInc, REAL(Theta,ncKind),  'VZA', 1, n_Theta  )
 CALL netCDF_Write_Var( UInc, REAL(Phi,ncKind),  'RAZ', 1, n_Phi    )
 CALL netCDF_Write_Var( UInc, REAL(Recep_Lev, ncKind), 'LOut', 1, Nrecep_Lev )
 CALL netCDF_Write_Var( UInc, I_Scenario, 'SSC' )
 CALL netCDF_Write_Var( UInc, REAL(Solar_Flux,ncKind),    'S0' )
 CALL netCDF_Write_Var( UInc, REAL(Surface_Temp,ncKind),    'TS' )

 StrLen = 6
 st2D(1:2) = 1
 ct2D(1)   = StrLen
 ct2D(2)   = nGas
 CALL netCDF_Write_Var( UInc, Gasname, 'Gas', st2D, ct2D )
 CALL netCDF_Write_Var( UInc, REAL(GasMu,ncKind), 'GasMW', 1, nGas )
 CALL netCDF_Write_Var( UInc, REAL(GasSF,ncKind), 'GasSF', 1, nGas )

 st2D(:) = 1
 ct2D(1) = nSpectra
 ct2D(2) = nMode
 CALL netCDF_Write_Var( UInc, REAL(NReal0(:,1:nMode),ncKind), 'MR', st2D, ct2D )
 CALL netCDF_Write_Var( UInc, REAL(NImag0(:,1:nMode),ncKind), 'MI', st2D, ct2D )

 CALL netCDF_Write_Var( UInc, REAL(ModeFrc(1:nMode),ncKind), 'ModeFrc', 1, nMode )
 CALL netCDF_Write_Var( UInc, IDist(1:nMode), 'PSDId', 1, nMode )
 CALL netCDF_Write_Var( UInc, IDProf(1:nMode), 'ProfId', 1, nMode )

 ct2D(1) = 2
 ct2D(2) = nMode
 CALL netCDF_Write_Var( UInc, REAL(SizeRange(:,1:nMode),ncKind), 'PSDRange', st2D, ct2D )
 CALL netCDF_Write_Var( UInc, REAL(Prof_Range(:,1:nMode),ncKind),'ProfRange', st2D, ct2D )
 CALL netCDF_Write_Var( UInc, REAL(Prof_Par(:,1:nMode),ncKind), 'ProfPars', st2D, ct2D )

 ct2D(1) = 3
 ct2D(2) = nMode
 CALL netCDF_Write_Var( UInc, REAL(DistPar(:,1:nMode),ncKind), 'PSDPars', st2D, ct2D )

 ! Variables for reference AOD
 IF ( L_Refer_AOD ) THEN 

    CALL netCDF_Write_Var( UInc, REAL(Refer_Lamda,ncKind), 'Ref_Lamda' )
    CALL netCDF_Write_Var( UInc, REAL(Refer_RMRI(1),ncKind), 'Ref_MR' )
    CALL netCDF_Write_Var( UInc, REAL(Refer_RMRI(2),ncKind), 'Ref_MI' )
    CALL netCDF_Write_Var( UInc, Refer_IDist, 'Ref_PSDId' )
    CALL netCDF_Write_Var( UInc, REAL(Refer_SizeRange,ncKind), 'Ref_PSDRange', 1, 2 )
    CALL netCDF_Write_Var( UInc, REAL(Refer_DistPar,ncKind), 'Ref_PSDPars', 1, 3 )

 ENDIF ! End of L_Refer_AOD

 ! Linarization parameters
 IF ( LVLIDORT .AND. LJacob .AND. N_LPar_Total >= 1 ) THEN 

    ! LinPar
    StrLen = 30
    st2D(1:2) = 1
    ct2D(1)   = StrLen
    ct2D(2)   = n_LPar_Total
    CALL netCDF_Write_Var( UInc, LPar_Names, 'LinPar', st2D, ct2D )
   
 ENDIF

!
 RETURN
!
 END SUBROUTINE WRITE_DIAG01
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG02( UInc )
!
! USES:
 USE NETCDF_MOD
 USE NAMELIST_ARRAY_MOD, ONLY : nGas
 USE NAMELIST_ARRAY_MOD, ONLY : nMode
 USE NAMELIST_ARRAY_MOD, ONLY : LAerosol
 USE ATMPROF_MOD,        ONLY : n_Layers
 USE ATMPROF_MOD,        ONLY : Atmos
 !USE ATMPROF_MOD,        ONLY : AtmZ, AtmP, AtmT
 !USE ATMPROF_MOD,        ONLY : LayP, LayT
 !USE ATMPROF_MOD,        ONLY : LayInt, LayMols
 USE AEROSOL_MOD,        ONLY : Prof_Vol, Prof_Num

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
!
! Local variables
 INTEGER            :: st1D, st2D(2)
 INTEGER            :: ct1D, ct2D(2)

!
! WRITE_DIAG02 begins here ...
!

 ! Z, P, T of each air level
 st1D = 1
 ct1D = n_Layers+1
 !CALL netCDF_Write_Var( UInc, REAL(AtmZ(1:n_Layers+1),ncKind),'Z', st1D, ct1D )
 !CALL netCDF_Write_Var( UInc, REAL(AtmP(1:n_Layers+1),ncKind),'P', st1D, ct1D )
 !CALL netCDF_Write_Var( UInc, REAL(AtmT(1:n_Layers+1),ncKind),'T', st1D, ct1D )
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Grid%Z(0:n_Layers),ncKind),'Z', st1D, ct1D )
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Grid%P(0:n_Layers),ncKind),'P', st1D, ct1D )
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Grid%T(0:n_Layers),ncKind),'T', st1D, ct1D )

 ! P, T, and air number density in each layer
 st1D = 1
 ct1D = n_Layers
 !CALL netCDF_Write_Var( UInc, REAL(LayP(1:n_Layers),ncKind),  'LayP',   st1D, ct1D )
 !CALL netCDF_Write_Var( UInc, REAL(LayT(1:n_Layers),ncKind),  'LayT',   st1D, ct1D )
 !CALL netCDF_Write_Var( UInc, REAL(LayInt(1:n_Layers),ncKind),'AirDen', st1D, ct1D )
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Box%P(1:n_Layers),ncKind), 'LayP', st1D, ct1D )
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Box%T(1:n_Layers),ncKind), 'LayT', st1D, ct1D )
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Box%AirDen(1:n_Layers),ncKind), 'AirDen', st1D, ct1D )

 ! Gas mixing ratio in each layer
 st2D = (/1, 1/)
 ct2D = (/n_Layers, nGas/)
 CALL netCDF_Write_Var( UInc, REAL(Atmos%Box%MixRatio(1:n_Layers,1:nGas),ncKind),'GasMR', st2D, ct2D )
 
 ! Only happens when Aerosol is considered by the model
 IF ( LAerosol ) THEN 
    st2D = (/1, 1/)
    ct2D = (/n_Layers, nMode/)
    CALL netCDF_Write_Var( UInc, REAL(Prof_Vol(1:n_Layers,1:nMode),ncKind),'AerVol', st2D, ct2D )
    CALL netCDF_Write_Var( UInc, REAL(Prof_Num(1:n_Layers,1:nMode),ncKind),'AerNum', st2D, ct2D )
 ENDIF

!
 RETURN
!
 END SUBROUTINE WRITE_DIAG02
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG03( UInc, iLam )
!
! USES:
 USE NETCDF_MOD
 USE NAMELIST_ARRAY_MOD, ONLY : n_Moments,     nMode
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_Refr,   LJacob_Size
 USE AEROSOL_MOD,        ONLY : PAngles
 USE AEROSOL_MOD,        ONLY : Asymm,         Bulk_AOP
 USE AEROSOL_MOD,        ONLY : ExpCoeffs,     FMatrix
 USE AEROSOL_MOD,        ONLY : Dist,          nPAngle
 USE AEROSOL_MOD,        ONLY : LPSD_Bulk,     LPSD_Dist
 USE AEROSOL_MOD,        ONLY : LRFE_BULK,     LFrc_Bulk
 USE AEROSOL_MOD,        ONLY : LPSD_FMatrix,  LRFE_FMatrix

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variables
 LOGICAL, SAVE       :: First = .True.

 INTEGER             :: st2D(2), st3D(3), st4D(4), st5D(5)
 INTEGER             :: ct2D(2), ct3D(3), ct4D(4), ct5D(5)
 
 REAL*4, ALLOCATABLE :: tmpArr2D(:,:)
 REAL*4, ALLOCATABLE :: tmpArr3D(:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr4D(:,:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr5D(:,:,:,:,:)

!
! WRITE_DIAG03 begins here ...
!

! PSD variables donot vary over spectra:
 IF ( iLam == 1 .OR. First ) THEN

    ! PAngle
    CALL netCDF_Write_Var( UInc, REAL(PAngles,ncKind), 'PAngle', 1, nPAngle )

    st2D = (/1, 1/)
    ct2D = (/5, nMode/)
    CALL netCDF_Write_Var( UInc, REAL(Dist,ncKind), 'Dist',st2D, ct2D )
 
    ! Only write it when PSD Jacobian calculation is performed by Mie
    IF ( LJacob_Size ) THEN 
       st3D = (/1, 1, 1/)  
       ct3D = (/5, 3, nMode/)
       CALL netCDF_Write_Var( UInc, REAL(LPSD_Dist, ncKind), 'LPSD_Dist', st3D, ct3D )
    ENDIF

    ! Change First to False
    First = .False.

 ENDIF

! Variables, which are function of spectra

 ! Bulk_AOP
 ALLOCATE( tmpArr3D(1,5,nMode) ) 
 tmpArr3D(1,:,:) = Bulk_AOP
 st3D = (/iLam, 1, 1/)
 ct3D = (/1, 5, nMode/)
 CALL netCDF_Write_Var( UInc, tmpArr3D, 'Bulk_AOP', st3D, ct3D )
 DEALLOCATE( tmpArr3D )

 ! Asymm
 ALLOCATE( tmpArr2D(1,nMode) )
 tmpArr2D(1,1:nMode) = Asymm(1:nMode)
 st2D = (/iLam, 1/)
 ct2D = (/1, nMode/)
 CALL netCDF_Write_Var( UInc, tmpArr2D, 'Asymm', st2D, ct2D )
 DEALLOCATE( tmpArr2D )

 ! ExpCoeffs
 ALLOCATE( tmpArr4D(1,6,n_Moments,nMode) )
 tmpArr4D(1,1:6,1:n_Moments,1:nMode) = ExpCoeffs(1:6,0:n_Moments-1,1:nMode)
 st4D = (/iLam, 1, 1, 1/)
 ct4D = (/1, 6, n_Moments, nMode/)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'ExpCoeffs', st4D, ct4D )
 DEALLOCATE( tmpArr4D )

 ! FMatrix
 ALLOCATE( tmpArr4D(1,6,nPAngle,nMode) )
 tmpArr4D(1,1:6,1:nPAngle,1:nMode) = FMatrix(1:6,1:nPAngle,1:nMode)
 st4D = (/iLam, 1, 1, 1/)
 ct4D = (/1, 6, nPAngle, nMode/)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'FMatrix', st4D, ct4D )
 DEALLOCATE( tmpArr4D )

 IF ( LJacob_Size ) THEN

    ! LPSD_Bulk
    ALLOCATE( tmpArr4D(1,5,3,nMode) )
    tmpArr4D(1,1:5,1:3,1:nMode) = LPSD_Bulk(1:5,1:3,1:nMode)
    st4D = (/iLam, 1, 1, 1/)
    ct4D = (/1, 5, 3, nMode/)
    CALL netCDF_Write_Var( UInc, tmpArr4D, 'LPSD_Bulk', st4D, ct4D )
    DEALLOCATE( tmpArr4D )

    ! LPSD_FMatrix
    ALLOCATE( tmpArr5D(1,6,nPAngle,3,nMode) )
    tmpArr5D(1,1:6,1:nPAngle,1:3,1:nMode) = LPSD_FMatrix(1:6,1:nPAngle,1:3,1:nMode)
    st5D = (/iLam, 1, 1, 1, 1/)
    ct5D = (/1, 6, nPAngle, 3, nMode/)
    CALL netCDF_Write_Var( UInc, tmpArr5D, 'LPSD_FMatrix', st5D, ct5D )
    DEALLOCATE( tmpArr5D )

 ENDIF

 IF ( LJacob_Refr ) THEN 

    ! LRFE_Bulk
    ALLOCATE( tmpArr4D(1,5,3,nMode) )
    tmpArr4D(1,1:5,1:3,1:nMode) = LRFE_Bulk(1:5,1:3,1:nMode)
    st4D = (/iLam, 1, 1, 1/)
    ct4D = (/1, 5, 3, nMode/)
    CALL netCDF_Write_Var( UInc, tmpArr4D, 'LRFE_Bulk', st4D, ct4D )
    DEALLOCATE( tmpArr4D )
  
    ! LRFE_FMatrix
    ALLOCATE( tmpArr5D(1,6,nPAngle,3,nMode) )
    tmpArr5D(1,1:6,1:nPAngle,1:3,1:nMode) = LRFE_FMatrix(1:6,1:nPAngle,1:3,1:nMode)
    st5D = (/iLam, 1, 1, 1, 1/)
    ct5D = (/1, 6, nPAngle, 3, nMode/)
    CALL netCDF_Write_Var( UInc, tmpArr5D, 'LRFE_FMatrix', st5D, ct5D )
    DEALLOCATE( tmpArr5D )

 ENDIF

 IF ( nMode == 2 ) THEN 

    ! LFrc_Bulk
    ALLOCATE( tmpArr3D(1,5,nMode) )
    tmpArr3D(1,:,:) = LFrc_Bulk
    st3D = (/iLam, 1, 1/)
    ct3D = (/1, 5, nMode/)
    CALL netCDF_Write_Var( UInc, tmpArr3D, 'LFrc_Bulk', st3D, ct3D )
    DEALLOCATE( tmpArr3D )

 ENDIF

!
 RETURN
!
 END SUBROUTINE WRITE_DIAG03
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG04( UInc, iLam )
!
! USES:
 USE NETCDF_MOD
 USE ATMPROF_MOD,        ONLY : n_Layers
 USE NAMELIST_ARRAY_MOD, ONLY : nGas,          nMode
 USE NAMELIST_ARRAY_MOD, ONLY : LTracegas,     LRayleigh
 USE NAMELIST_ARRAY_MOD, ONLY : LAerosol
 USE NAMELIST_ARRAY_MOD, ONLY : LContinuum
 USE SPECTRA_MOD,        ONLY : nSpectra
 USE GAS_MOD,            ONLY : XSec_SAO,      XSec_HIT
 USE GAS_MOD,            ONLY : XAbs_SAO,      XAbs_HIT
 USE GAS_MOD,            ONLY : XSec_CTN,      XAbs_CTN
 USE GAS_MOD,            ONLY : Gas_XSec,      Gas_XAbs
 USE RAYLEIGH_MOD,       ONLY : Rayleigh_XSca, Rayleigh_Phas
 USE AIROPTIC_MOD,       ONLY : TauAer,        SSAAer

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variables
 LOGICAL, SAVE       :: First = .True.

 INTEGER             :: st2D(2), st3D(3)
 INTEGER             :: ct2D(2), ct3D(3)

 REAL*4, ALLOCATABLE :: tmpArr2D(:,:)
 REAL*4, ALLOCATABLE :: tmpArr3D(:,:,:)

!
! WRITE_DIAG04 begins here ...
!
 
! PSD variables donot vary over spectra or calculation is done before spectra
! loop
 IF ( iLam == 1 .OR. First ) THEN

   ! Tracegas variables
    IF ( LTracegas ) THEN 

       ! Xsec_SAO, Xsec_LBL, Xsec_Combined
       ! tauGas_SAO, tauGas_LBL, tauGas_Combined
       st3D = (/1, 1, 1/)
       ct3D = (/nSpectra, n_Layers, nGas/)
       CALL netCDF_Write_Var( UInc, REAL(Xsec_SAO,ncKind), 'Xsec_SAO', st3D, ct3D )
       CALL netCDF_Write_Var( UInc, REAL(Xsec_HIT,ncKind), 'Xsec_LBL', st3D, ct3D )
       CALL netCDF_Write_Var( UInc, REAL(Gas_XSec,ncKind), 'Xsec_Combined', st3D, ct3D )
       CALL netCDF_Write_Var( UInc, REAL(XAbs_SAO,ncKind), 'tauGas_SAO', st3D, ct3D )
       CALL netCDF_Write_Var( UInc, REAL(XAbs_HIT,ncKind), 'tauGas_LBL', st3D, ct3D )
       IF ( LContinuum ) THEN 
          CALL netCDF_Write_Var( UInc, REAL(Xsec_CTN,ncKind), 'Xsec_CTN', st3D, ct3D )
          CALL netCDF_Write_Var( UInc, REAL(XAbs_CTN,ncKind), 'tauGas_CTN', st3D, ct3D )
       ENDIF ! LContinuum

       CALL netCDF_Write_Var( UInc, REAL(Gas_XAbs,ncKind), 'tauGas_Combined', st3D, ct3D )

       ! tauGas
       ALLOCATE( tmpArr2D(nSpectra, n_Layers ) )
       tmpArr2D = SUM( Gas_XAbs, 3 )
       st2D = (/1, 1/)
       ct2D = (/nSpectra, n_Layers/)
       CALL netCDF_Write_Var( UInc, tmpArr2D, 'tauGas', st2D, ct2D )
       DEALLOCATE( tmpArr2D )

    ENDIF

    ! Rayleigh Scattering
    IF ( LRayleigh ) THEN 

       ! tauRayleigh
       st2D = (/1, 1/)
       ct2D = (/nSpectra, n_Layers/)
       CALL netCDF_Write_Var( UInc, REAL(RAYLEIGH_XSCA,ncKind), 'tauRayleigh', st2D, ct2D )  

       ! Rayleigh_FMatrix
       st3D = (/1, 1, 1/)
       ct3D = (/nSpectra, 3, 6/)
       CALL netCDF_Write_Var( UInc, REAL(Rayleigh_Phas,ncKind), 'Rayleigh_FMatrix', st3D, ct3D )

    ENDIF

    ! Change First to False
    First = .False.

 ENDIF

! Aerosol scattering are calculated within the spectra loop
 IF ( LAerosol ) THEN 

    st3D = (/iLam, 1, 1/)
    ct3D = (/1, n_Layers, nMode/)

    ! tauAER
    ALLOCATE( tmpArr3D(1,n_Layers,nMode) )
    tmpArr3D(1,:,:) = tauAer
    CALL netCDF_Write_Var( UInc, tmpArr3D, 'tauAER', st3D, ct3D )
    DEALLOCATE( tmpArr3D )

    ! omegaAER 
    ALLOCATE( tmpArr3D(1,n_Layers,nMode) )
    tmpArr3D(1,:,:) = SSAAer
    CALL netCDF_Write_Var( UInc, tmpArr3D, 'omegaAER', st3D, ct3D )
    DEALLOCATE( tmpArr3D )

 ENDIF

 END SUBROUTINE WRITE_DIAG04
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG05( UInc, iLam ) 
!
! USES:
 USE NETCDF_MOD
 USE NAMELIST_ARRAY_MOD, ONLY : LBRDF, LLambertian, LSLeave
 USE NAMELIST_ARRAY_MOD, ONLY : n_BRDF, BRDF_Names, BRDF_Factor
 USE NAMELIST_ARRAY_MOD, ONLY : n_Stoks
 USE NAMELIST_ARRAY_MOD, ONLY : n_Theta, n_Theta0, n_Phi
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_Surf
 USE SPECTRA_MOD,        ONLY : nSpectra
 USE BRDF_MOD,           ONLY : BRDFVal, BRDFJCB
 USE SLEAVE_MOD,         ONLY : RSL_Iso
 USE LAMB_ALBEDO_MOD,    ONLY : Lamb_Albedo
 USE LPAR_MOD,           ONLY : N_LPAR_SURF

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variable 
 LOGICAL, SAVE       :: First = .True.

 INTEGER             :: st1D, st2D(2), st3D(3), st5D(5), st6D(6)
 INTEGER             :: ct1D, ct2D(2), ct3D(3), ct5D(5), ct6D(6)
 INTEGER             :: StrLen
 INTEGER             :: n_Stoks2

 REAL*4, ALLOCATABLE :: tmpArr1D(:)
 REAL*4, ALLOCATABLE :: tmpArr2D(:,:)
 REAL*4, ALLOCATABLE :: tmpArr3D(:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr5D(:,:,:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr6D(:,:,:,:,:,:)

 IF ( LLambertian ) THEN 
 
    ! Lamb_Albedo
    IF ( iLam == 1 .OR. First ) THEN 
       CALL netCDF_Write_Var( UInc, REAL(Lamb_Albedo,ncKind), 'Lamb_Albedo', 1, nSPectra )
       First = .False.
    ENDIF

 ENDIF ! LLambertian

 IF ( LBRDF ) THEN 

    IF ( iLam == 1 .OR. First ) THEN
       ! BRDFKernel 
       StrLen = 10
       st2D(1:2) = 1
       ct2D(1)   = StrLen
       ct2D(2)   = n_BRDF
       CALL netCDF_Write_Var( UInc, BRDF_Names(1:n_BRDF), 'BRDFKernel', st2D, ct2D )
       First = .False.
    ENDIF

    ! BRDFFactor
    st2D = (/ iLam, 1 /)
    ct2D = (/ 1, n_BRDF /)
    ALLOCATE( tmpArr2D(1, n_BRDF) )
    tmpArr2D(1,:) = BRDF_Factor(1:n_BRDF)
    CALL netCDF_Write_Var( UInc, tmpArr2D, 'BRDFFactor', st2D, ct2D )
    DEALLOCATE( tmpArr2D )
  
    IF ( n_Stoks == 1 ) THEN 
       n_Stoks2 = 1
    ELSE
       n_Stoks2 = 16
    ENDIF

    ! BRDFVal
    st5D = (/ iLam, 1, 1, 1, 1 /)
    ct5D = (/ 1, n_Stoks2, n_Theta, n_Phi, n_Theta0 /)
    ALLOCATE( tmpArr5D(1, n_Stoks2, n_Theta, n_Phi, n_Theta0) )
    tmpArr5D(1,:,:,:,:) = BRDFVAL(1:n_Stoks2,:,:,:)
    CALL netCDF_Write_Var( UInc, tmpArr5D, 'BRDFVal', st5D, ct5D )
    DEALLOCATE( tmpArr5D )

    ! BRDFJacob
    IF ( LJacob_Surf ) THEN 

       st6D = (/ iLam, 1, 1, 1, 1, 1 /)
       ct6D = (/ 1, N_LPAR_SURF, n_Stoks2, n_Theta, n_Phi, n_Theta0 /)
       ALLOCATE( tmpArr6D(1, N_LPAR_SURF, n_Stoks2, n_Theta, n_Phi, n_Theta0) )
       tmpArr6D(1,:,:,:,:,:) = BRDFJCB(1:N_LPAR_SURF,1:n_Stoks2,:,:,:)
       CALL netCDF_Write_Var( UInc, tmpArr6D, 'BRDFJacob', st6D, ct6D )
       DEALLOCATE( tmpArr6D )
      
    ENDIF

 ENDIF ! LBRDF

 IF ( LSleave ) THEN
  
    ! SLeaveVal
    st3D = (/ iLam, 1, 1 /)
    ct3D = (/ 1, n_Stoks, n_Theta0 /)
    ALLOCATE( tmpArr3D(1, n_Stoks, n_Theta0) )
    tmpArr3D(1,:,:) = RSL_Iso(1:n_Stoks,1:n_Theta0)
    CALL netCDF_Write_Var( UInc, tmpArr3D, 'SLeaveVal', st3D, ct3D )
    DEALLOCATE( tmpArr3D )

 ENDIF ! SLeave

!
 RETURN
!
 END SUBROUTINE WRITE_DIAG05
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG06( UInc, iLam )
!
! USES:
 USE NETCDF_MOD
 USE ATMPROF_MOD,        ONLY : n_Layers
 USE LPAR_MOD,           ONLY : n_LPar_Total
 USE IOP_MOD,            ONLY : nGMMask, GMMask
 USE IOP_MOD,            ONLY : IOP_Tau,  IOP_SSA,  IOP_Pha
 USE IOP_MOD,            ONLY : LIOP_Tau, LIOP_SSA, LIOP_Pha
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob, n_Moments

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variable 
 LOGICAL, SAVE       :: First = .True.
 INTEGER             :: st2D(2), st3D(3), st4D(4), st5D(5)
 INTEGER             :: ct2D(2), ct3D(3), ct4D(4), ct5D(5)

 REAL*4, ALLOCATABLE :: tmpArr2D(:,:)
 REAL*4, ALLOCATABLE :: tmpArr3D(:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr4D(:,:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr5D(:,:,:,:,:)
!
! WRITE_DIAG06 begins here ....
!
 IF ( iLam == 1 .OR. First ) THEN
    CALL netCDF_Write_Var( UInc, GMMask, 'GMat', 1, nGMMask )
    First = .False.
 ENDIF

    ! tauIOP and omegaIOP
    st2D = (/iLam, 1/)
    ct2D = (/1, n_Layers/)

    ALLOCATE( tmpArr2D(1,n_Layers) )
    tmpArr2D(1,:) = IOP_Tau
    CALL netCDF_Write_Var( UInc, tmpArr2D, 'tauIOP', st2D, ct2D )
    DEALLOCATE( tmpArr2D )

    ALLOCATE( tmpArr2D(1,n_Layers) )
    tmpArr2D(1,:) = IOP_SSA
    CALL netCDF_Write_Var( UInc, tmpArr2D, 'omegaIOP', st2D, ct2D )
    DEALLOCATE( tmpArr2D )

    ! GMatIOP
    st4D = (/iLam, 1, 1, 1/)
    ct4D = (/1, n_Moments, n_Layers, nGMMask/)
    ALLOCATE( tmpArr4D(1, n_Moments, n_Layers, nGMMask) )
    tmpArr4D(1,:,:,:) = IOP_Pha(0:n_Moments-1,1:n_Layers,1:nGMMask)
    CALL netCDF_Write_Var( UInc, tmpArr4D, 'GMatIOP', st4D, ct4D )
    DEALLOCATE( tmpArr4D )

    ! Linearized inputs
    IF ( LJacob .AND. n_LPar_Total >= 1 ) THEN 

       ! tauLIOP and omegaLIOP
       st3D = (/iLam, 1, 1/)
       ct3D = (/1, n_LPar_Total, n_Layers/)
    
       ALLOCATE( tmpArr3D(1,n_LPar_Total,n_Layers) )
       tmpArr3D(1,:,:) = LIOP_Tau
       CALL netCDF_Write_Var( UInc, tmpArr3D, 'tauLIOP', st3D, ct3D )
       DEALLOCATE( tmpArr3D )

       ALLOCATE( tmpArr3D(1,n_LPar_Total,n_Layers) )
       tmpArr3D(1,:,:) = LIOP_SSA
       CALL netCDF_Write_Var( UInc, tmpArr3D, 'omegaLIOP', st3D, ct3D )
       DEALLOCATE( tmpArr3D )

       ! GMatLIOP
       st5D = (/iLam, 1, 1, 1, 1/)
       ct5D = (/1, n_LPar_Total, n_Moments, n_Layers, nGMMask/)
       ALLOCATE( tmpArr5D(1, n_LPar_Total, n_Moments, n_Layers, nGMMask) )
       tmpArr5D(1,:,:,:,:) = LIOP_Pha(1:n_LPar_Total,0:n_Moments-1,1:n_Layers,1:nGMMask)
       CALL netCDF_Write_Var( UInc, tmpArr5D, 'GMatLIOP', st5D, ct5D )
       DEALLOCATE( tmpArr5D ) 

    ENDIF

!
 RETURN
!
 END SUBROUTINE WRITE_DIAG06
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG07( UInc, iLam )
!
! USES:
 USE NETCDF_MOD
 USE ATMPROF_MOD,        ONLY : n_Layers
 USE NAMELIST_ARRAY_MOD, ONLY : LEmiss_Atmos, LEmiss_Surface
 USE NAMELIST_ARRAY_MOD, ONLY : n_Theta0, n_Theta
 USE NAMELIST_ARRAY_MOD, ONLY : n_Stoks, nRecep_Lev, n_Geom
 USE VLIDORT_ARRAY_MOD,  ONLY : VZA_Offset
 USE VLIDORT_ARRAY_MOD,  ONLY : Stokes_Comps
 USE VLIDORT_ARRAY_MOD,  ONLY : Mean_Stokes
 USE VLIDORT_ARRAY_MOD,  ONLY : Flux_Stokes
 USE VLIDORT_ARRAY_MOD,  ONLY : Mean_Direct
 USE VLIDORT_ARRAY_MOD,  ONLY : Flux_Direct
 USE VLIDORT_ARRAY_MOD,  ONLY : Emiss_Surface
 USE VLIDORT_ARRAY_MOD,  ONLY : GDT_Emiss_Surface
 USE VLIDORT_ARRAY_MOD,  ONLY : Emiss_Atmos
 USE VLIDORT_ARRAY_MOD,  ONLY : GDT_Emiss_Atmos

!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variable 
 LOGICAL, SAVE       :: First = .True.

 INTEGER             :: st1D, st2D(2), st4D(4)
 INTEGER             :: ct1D, ct2D(2), ct4D(4)

 REAL*4, ALLOCATABLE :: tmpArr1D(:)
 REAL*4, ALLOCATABLE :: tmpArr2D(:,:)
 REAL*4, ALLOCATABLE :: tmpArr4D(:,:,:,:)

!
! WRITE_DIAG07 begins here ....
!

! Geo_Offset
 IF ( iLam == 1 .OR. First ) THEN
    st2D = (/1, 1/)
    ct2D = (/n_Theta0, n_Theta/)
    CALL netCDF_Write_Var( UInc, REAL(VZA_Offset,ncKind), 'Geo_Offset', st2D, ct2D )
    First = .False.
 ENDIF 

! Stokes 
 ALLOCATE( tmpArr4D(1,nRecep_Lev,n_Geom,n_Stoks) )
 tmpArr4D(1,:,:,:) = Stokes_Comps(1:nRecep_Lev,1:n_Geom,1:n_Stoks)
 st4D = (/iLam, 1, 1, 1/)
 ct4D = (/1, nRecep_Lev,n_Geom,n_Stoks/)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'Stokes', st4D, ct4D )
 DEALLOCATE( tmpArr4D )

! Mean_Stokes, Flux_Stokes, Mean_Direct, Flux_Direct
 st4D = (/iLam, 1, 1, 1/)
 ct4D = (/1, nRecep_Lev,n_Theta0,n_Stoks/)

 ALLOCATE( tmpArr4D(1,nRecep_Lev,n_Theta0,n_Stoks) )
 tmpArr4D(1,:,:,:) = Mean_Stokes(1:nRecep_Lev,1:n_Theta0,1:n_Stoks)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'Mean_Stokes', st4D, ct4D )
 DEALLOCATE( tmpArr4D )
 
 ALLOCATE( tmpArr4D(1,nRecep_Lev,n_Theta0,n_Stoks) )
 tmpArr4D(1,:,:,:) = Flux_Stokes(1:nRecep_Lev,1:n_Theta0,1:n_Stoks)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'Flux_Stokes', st4D, ct4D )
 DEALLOCATE( tmpArr4D )

 ALLOCATE( tmpArr4D(1,nRecep_Lev,n_Theta0,n_Stoks) )
 tmpArr4D(1,:,:,:) = Mean_Direct(1:nRecep_Lev,1:n_Theta0,1:n_Stoks)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'Mean_Direct', st4D, ct4D )
 DEALLOCATE( tmpArr4D )

 ALLOCATE( tmpArr4D(1,nRecep_Lev,n_Theta0,n_Stoks) )
 tmpArr4D(1,:,:,:) = Flux_Direct(1:nRecep_Lev,1:n_Theta0,1:n_Stoks)
 CALL netCDF_Write_Var( UInc, tmpArr4D, 'Flux_Direct', st4D, ct4D )
 DEALLOCATE( tmpArr4D )

 ! Atmosphere emissions
 IF ( LEmiss_Atmos ) THEN

    ! Emiss_Atmos and GDT_Emiss_Atmos
    st2D = (/iLam, 1/)
    ct2D = (/1, n_Layers+1/)

    ALLOCATE( tmpArr2D(1,n_Layers+1) )
    tmpArr2D(1,:) = Emiss_Atmos(0:n_Layers)
    CALL netCDF_Write_Var( UInc, tmpArr2D, 'Emiss_Atmos', st2D, ct2D )
    DEALLOCATE( tmpArr2D )
  
    ALLOCATE( tmpArr2D(1,n_Layers+1) )
    tmpArr2D(1,:) = GDT_Emiss_Atmos(0:n_Layers)
    CALL netCDF_Write_Var( UInc, tmpArr2D, 'GDT_Emiss_Atmos', st2D, ct2D )
    DEALLOCATE( tmpArr2D )

 ENDIF

! Surface emissions
 IF ( LEmiss_Surface ) THEN

    ! Emiss_Surface and GDT_Emiss_Surface
    st1D = iLam
    ct1D = 1

    ALLOCATE( tmpArr1D(1) )
    tmpArr1D(1) = Emiss_Surface
    CALL netCDF_Write_Var( UInc, tmpArr1D, 'Emiss_Surface', st1D, ct1D )
    DEALLOCATE( tmpArr1D )

    ALLOCATE( tmpArr1D(1) )
    tmpArr1D(1) = GDT_Emiss_Surface
    CALL netCDF_Write_Var( UInc, tmpArr1D, 'GDT_Emiss_Surface', st1D, ct1D )
    DEALLOCATE( tmpArr1D )

 ENDIF

!
 RETURN
!
END SUBROUTINE WRITE_DIAG07
!
!------------------------------------------------------------------------------
!
 SUBROUTINE WRITE_DIAG08( UInc, iLam )
!
! USES:
 USE NETCDF_MOD
 USE ATMPROF_MOD,        ONLY : n_Layers
 USE NAMELIST_ARRAY_MOD, ONLY : n_Stoks, nRecep_Lev, n_Geom, nGas
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_Surf, LAerosol
 USE NAMELIST_ARRAY_MOD, ONLY : LJacob_Gas,  LJacob_C, LJacob_P
 USE LPAR_MOD,           ONLY : n_LPar_Total
 USE LPAR_MOD,           ONLY : n_LPar_Surf
 USE VLIDORT_ARRAY_MOD,  ONLY : Jacks_Profls, Jacks_Column
 USE VLIDORT_ARRAY_MOD,  ONLY : Jacks_Surf,   Jacks_Gases
 USE IOP_MOD,            ONLY : TauAer_WFs
!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)        :: UInc
 INTEGER, INTENT(IN)        :: iLam

! Local variable 
 INTEGER             :: st3D(3), st5D(5), st6D(6)
 INTEGER             :: ct3D(3), ct5D(5), ct6D(6)

 REAL*4, ALLOCATABLE :: tmpArr3D(:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr5D(:,:,:,:,:)
 REAL*4, ALLOCATABLE :: tmpArr6D(:,:,:,:,:,:)

!
! WRITE_DIAG07 begins here ....
!

  ! Atmospheric Jacobians
  IF ( n_LPar_Total >= 1) THEN 

     ! Jacob_Column
     IF ( LJacob_C ) THEN 
     st5D = (/iLam, 1, 1, 1, 1/)
     ct5D = (/1, n_LPar_Total, nRecep_Lev, n_Geom, n_Stoks/)
     ALLOCATE( tmpArr5D(1, n_LPar_Total, nRecep_Lev, n_Geom, n_Stoks) )
     tmpArr5D(1,:,:,:,:) = Jacks_Column(:,:,:,:)
     CALL netCDF_Write_Var( UInc, tmpArr5D, 'Jacob_Column', st5D, ct5D )
     DEALLOCATE( tmpArr5D )
     ENDIF

     ! Jacob_Prof
     IF ( LJacob_P ) THEN 
     st6D = (/iLam, 1, 1, 1, 1, 1/)
     ct6D = (/1, n_LPar_Total, n_Layers, nRecep_Lev, n_Geom, n_Stoks/)
     ALLOCATE( tmpArr6D(1, n_LPar_Total, n_Layers, nRecep_Lev, n_Geom, n_Stoks) )
     tmpArr6D(1,:,:,:,:,:) = Jacks_Profls(:,:,:,:,:)
     CALL netCDF_Write_Var( UInc, tmpArr6D, 'Jacob_Prof', st6D, ct6D )
     DEALLOCATE( tmpArr6D )
     ENDIF

     ! Jacob_Prof_Gas
     IF ( LJacob_P .AND. LJacob_Gas ) THEN
     st6D = (/iLam, 1, 1, 1, 1, 1/)
     ct6D = (/1, nGas, n_Layers, nRecep_Lev, n_Geom, n_Stoks/)
     ALLOCATE( tmpArr6D(1, nGas, n_Layers, nRecep_Lev, n_Geom, n_Stoks) )
     tmpArr6D(1,:,:,:,:,:) = Jacks_Gases(:,:,:,:,:)
     CALL netCDF_Write_Var( UInc, tmpArr6D, 'Jacob_Prof_Gas', st6D, ct6D )
     DEALLOCATE( tmpArr6D )
     ENDIF

     ! tauAER_WFS
     IF ( LAerosol ) THEN  

        st3D = (/iLam, 1, 1/)
        ct3D = (/1, n_LPar_Total, n_Layers/)
        ALLOCATE( tmpArr3D(1, n_LPar_Total, n_Layers ) )
        tmpArr3D(1,:,:) = TauAer_WFS(:,:)
        CALL netCDF_Write_Var( UInc, tmpArr3D, 'tauAER_WFS', st3D, ct3D )
        DEALLOCATE( tmpArr3D )

     ENDIF

 ENDIF

! Jacob_Surface
 IF ( LJacob_Surf .AND. n_LPar_Surf >= 1 ) THEN 

     st5D = (/iLam, 1, 1, 1, 1/)
     ct5D = (/1, n_LPar_Surf, nRecep_Lev, n_Geom, n_Stoks/)
     ALLOCATE( tmpArr5D(1, n_LPar_Surf, nRecep_Lev, n_Geom, n_Stoks) )
     tmpArr5D(1,:,:,:,:) = Jacks_Surf(:,:,:,:)
     CALL netCDF_Write_Var( UInc, tmpArr5D, 'Jacob_Surface', st5D, ct5D )
     DEALLOCATE( tmpArr5D )

 ENDIF

!
 RETURN
!
 END SUBROUTINE WRITE_DIAG08
!
!------------------------------------------------------------------------------
!
 SUBROUTINE CLOSEUP_DIAG( UInc )
!
! USES:
 USE NETCDF_MOD
!
 IMPLICIT NONE
!
! Arguments:
 INTEGER, INTENT(IN)         :: UInc
!
! Local variables
 INTEGER                     :: vID

! Go to define mode
 CALL netCDF_Begin_Def ( UInc )
  
! Write one last attribute to indicate a sucessful write
 vID = -999
 CALL netCDF_Def_Attr( UInc, vID, 'Yes', 'Scucess' )

! Close the netCDF file and the diaginfo file
 CALL netCDF_Close( UInc )

 END SUBROUTINE CLOSEUP_DIAG
!
!------------------------------------------------------------------------------
!
 SUBROUTINE CREATE_DIAGINFO ( IUinfo, infoFile, ncFile )
!
! USES:
 USE ERROR_MOD,          ONLY : ERROR_STOP
 USE TIME_MOD,           ONLY : SYSTEM_TIMESTAMP 
!
 IMPLICIT NONE
!
! Arguments
 INTEGER, INTENT(IN)         :: IUinfo
 CHARACTER(LEN=*), INTENT(IN):: infoFile
 CHARACTER(LEN=*), INTENT(IN):: ncFile
!
! Local Variables
 CHARACTER(LEN=128)          :: Msg
 CHARACTER(LEN=16)           :: Stamp
 INTEGER                     :: ios
!
! CREATE_DIAGINFO begins here ...
!
! Create a text file
 OPEN( UNIT=IUinfo, FILE=infoFile, STATUS='replace', ACTION='write', IOSTAT=ios )
 print*, 'ios = ', ios
 IF ( ios /= 0 ) THEN 
    Msg = 'Cannot open the file: ' // infoFile
    CALL ERROR_STOP( Msg, 'diag_mod.f90' )
 ENDIF

! Write file header
 Stamp = SYSTEM_TIMESTAMP()
 WRITE(IUinfo,'(A)') '#' // REPEAT( '=', 78 )
 WRITE(IUinfo,  100) Stamp
 WRITE(IUinfo,  105) 
 WRITE(IUinfo,  110) TRIM( ncFile )
 WRITE(IUinfo,  115)
 WRITE(IUinfo,'(A)') '#' // REPEAT( '=', 78 )

 100 FORMAT( 1X, 'This file is created by UNL-VRTM at ', A )
 105 FORMAT( 1X, 'It contains info for variables in the output netCDF file:' )
 110 FORMAT( 3X, '"', A, '"', / )
 115 FORMAT( 1X, ' Name:     Units:     Longname:' ) 

 END SUBROUTINE CREATE_DIAGINFO


END MODULE DIAG_MOD
